#include "dmn.h"

void
usage (char *app)
{
  printf ("\n%s - firewall log watch daemon\n", app);
  printf ("version %.1f\nIan Jones - roux@speakeasy.org\n\n", VERSION);
  printf ("   Usage: %s [-k|-d]\n\n", app);
  printf ("     -d: \"daemon\" - Become a daemon process.\n");
  printf ("     -k: \"kill\" - Kill a living daemon.\n\n");
  printf ("Use without any options to print to STDOUT.\n\n");
  exit (1);
}


int
sig_catch (int catch)
{
  if (catch == CATCH)
    {
      if (signal (SIGINT, quit) == SIG_IGN)
	signal (SIGINT, SIG_IGN);
      if (signal (SIGHUP, quit) == SIG_IGN)
	signal (SIGHUP, SIG_IGN);
      if (signal (SIGTERM, quit) == SIG_IGN)
	signal (SIGTERM, SIG_IGN);
      if (signal (SIGPIPE, quit) == SIG_IGN)
	signal (SIGPIPE, SIG_IGN);
      if (signal (SIGQUIT, quit) == SIG_IGN)
	signal (SIGQUIT, SIG_IGN);
    }
  return 0;
}

int
highlander (char *opt)
{
  int pid;
  FILE *pidfile;
  if ((pidfile = fopen (PIDFILE, "r")) != NULL)
    {
      fscanf (pidfile, "%d", &pid);
      fclose (pidfile);
    }
  else
    pid = 0;
  if (pid > 0 && (strcmp (opt, "-d") == 0))
    return 1;
  if (pid > 0 && (strcmp (opt, "STDOUT") == 0))
    return 1;
  if (pid > 0 && (strcmp (opt, "-k") == 0))
    {
      int death;
      death = kill (pid, SIGINT);
      if (death == 0)
	{
	  printf ("Killed %d.\n", pid);
	  unlink (PIDFILE);
	}
      else
	{
	  printf ("Stale PID. Removing %s.\n", PIDFILE);
	  unlink (PIDFILE);
	}
      exit (1);
    }
  if (pid + strcmp (opt, "-k") == 0)
    {
      printf ("Nothing to kill!\n");
      exit (1);
    }
  return 0;
}

void
quit (int signum)
{
  extern FILE *logst;
  int pid;
  FILE *pidfile;
  fclose (logst);
  writebuf ();
  if ((pidfile = fopen (PIDFILE, "r")) != NULL)
    {
      fscanf (pidfile, "%d", &pid);
      fclose (pidfile);
    }
  unlink (PIDFILE);
  exit (signum);
}


int
readbuf (void)
{
  FILE *buffer;
  extern int buf_counter;
  int read = 0;
  extern struct pktlog mailbuffer[];
  if ((buffer = fopen (BUFFERFILE, "r")) != NULL)
    {
      read = fread (mailbuffer, sizeof (struct pktlog), BUFFERSIZE, buffer);
      buf_counter = read - 1;
    }
  unlink (BUFFERFILE);
  return read;
}

int
writebuf (void)
{
  FILE *buffer;
  extern int buf_counter;
  extern int tailer;
  extern struct pktlog mailbuffer[];
  int wrote=0;
  if (tailer)
    return 0;
  if ((buffer = fopen (BUFFERFILE, "w")) == NULL)
    {
      return 0;
    }
  else
    {
      if (buf_counter != 0)
	{
	  wrote =
	    fwrite (mailbuffer, sizeof (struct pktlog), buf_counter, buffer);
	}
    }
  fclose (buffer);
  return wrote;
}

int
fmt_prn (FILE * out, struct pktlog *pkt)
{
  int printed;
  if (strcmp (pkt->proto, "icmp") == 0)
    {
      printed =
	(fprintf
	 (out, "%s  %s  LOG: %s\nTYPE:%d/CODE:%d - %s\n%s -> %s\n%s\n\n",
	  pkt->date, pkt->proto, pkt->msg, pkt->icmptype, pkt->icmpcode,
	  pkt->icmp_res, pkt->srcip, pkt->dstip, pkt->srcip_res));
    }
  else
    {
      printed =
	(fprintf
	 (out, "%s  %s  LOG: %s\n%s:%d -> %s:%d\n%s -> %s\n\n", pkt->date,
	  pkt->proto, pkt->msg, pkt->srcip, pkt->srcpt, pkt->dstip,
	  pkt->dstpt, pkt->srcip_res, pkt->dstpt_res));
    }
  return printed;
}
