#include "dmn.h"
/* There has to be a better way!
 * The following function holds every possible ICMP type/code in memory
 * at all times. This seems like a lot of work for the occasional
 * ICMP lookup that is probably harmless anyway.
 * http://www.isi.edu/in-notes/iana/assignments/icmp-parameters
 */
int
icmp_resolve (int icmptype, int icmpcode, char *icmpst)
{
  char restype[64];
  char codetype[64];

  switch (icmptype)
    {
    case 0:
      strcpy (restype, "Echo Reply");
      strcpy (codetype, "No Code");
      break;
    case 3:
      strcpy (restype, "Destination Unreachable");
      switch (icmpcode)
	{
	case 0:
	  strcpy (codetype, "Net Unreachable");
	  break;
	case 1:
	  strcpy (codetype, "Host Unreachable");
	  break;
	case 2:
	  strcpy (codetype, "Protocol Unreachable");
	  break;
	case 3:
	  strcpy (codetype, "Port Unreachable");
	  break;
	case 4:
	  strcpy (codetype,
		  "Fragmentation Needed and Don't Fragment was Set");
	  break;
	case 5:
	  strcpy (codetype, "Source Route Failed");
	  break;
	case 6:
	  strcpy (codetype, "Destination Network Unknown");
	  break;
	case 7:
	  strcpy (codetype, "Destination Host Unknown");
	  break;
	case 8:
	  strcpy (codetype, "Source Host Isolated");
	  break;
	case 9:
	  strcpy (codetype,
		  "Destination Network Administratively Prohibited");
	  break;
	case 10:
	  strcpy (codetype, "Destination Host Administratively Prohibited");
	  break;
	case 11:
	  strcpy (codetype,
		  "Destination Network Unreachable for Type of Service");
	  break;
	case 12:
	  strcpy (codetype,
		  "Destination Host Unreachable for Type of Service");
	  break;
	case 13:
	  strcpy (codetype, "Communication Administratively Prohibited");
	  break;
	case 14:
	  strcpy (codetype, "Host Precedence Violation");
	  break;
	case 15:
	  strcpy (codetype, "Precedence cutoff in effect");
	  break;
	default:
	  strcpy (codetype, "unknown code");

	}
      break;
    case 4:
      strcpy (restype, "Source Quench");
      strcpy (codetype, "No Code");
      break;
    case 5:
      strcpy (restype, "Redirect");
      switch (icmpcode)
	{
	case 0:
	  strcpy (codetype, "Redirect Datagram for the Network (or subnet)");
	  break;
	case 1:
	  strcpy (codetype, "Redirect Datagram for the Host");
	  break;
	case 2:
	  strcpy (codetype,
		  "Redirect Datagram for the Type of Service and Network");
	  break;
	case 3:
	  strcpy (codetype,
		  "Redirect Datagram for the Type of Service and Host");
	  break;
	default:
	  strcpy (codetype, "unknown code");
	}
      break;
    case 6:
      strcpy (restype, "Alternate Host Address");
      if (icmpcode == 0)
	strcpy (codetype, "Alternate Address for Host");
      else
	strcpy (codetype, "unknown code");
      break;
    case 8:
      strcpy (restype, "Echo");
      if (icmpcode == 0)
	strcpy (codetype, "No Code");
      else
	strcpy (codetype, "unknown code");
      break;
    case 9:
      strcpy (restype, "Router Advertisement");
      if (icmpcode == 0)
	strcpy (codetype, "No Code");
      else
	strcpy (codetype, "unknown code");
      break;
    case 10:
      strcpy (restype, "Router Selection");
      if (icmpcode == 0)
	strcpy (codetype, "No Code");
      else
	strcpy (codetype, "unknown code");
      break;
    case 11:
      strcpy (restype, "Time Exceeded");
      switch (icmpcode)
	{
	case 0:
	  strcpy (codetype, "Time to Live exceeded in Transit");
	  break;
	case 1:
	  strcpy (codetype, "Fragment Reassembly Time Exceeded");
	  break;
	default:
	  strcpy (codetype, "unknown code");
	}
      break;
    case 12:
      strcpy (restype, "Parameter Problem");
      switch (icmpcode)
	{
	case 0:
	  strcpy (codetype, "Pointer indicates the error");
	  break;
	case 1:
	  strcpy (codetype, "Missing a Required Option");
	  break;
	case 2:
	  strcpy (codetype, "Bad Length");
	  break;
	default:
	  strcpy (codetype, "unknown code");
	}
      break;
    case 13:
      strcpy (restype, "Timestamp");
      if (icmpcode == 0)
	strcpy (codetype, "No Code");
      else
	strcpy (codetype, "unknown code");
      break;
    case 14:
      strcpy (restype, "Timestamp Reply");
      if (icmpcode == 0)
	strcpy (codetype, "No Code");
      else
	strcpy (codetype, "unknown code");
      break;
    case 15:
      strcpy (restype, "Information Request");
      if (icmpcode == 0)
	strcpy (codetype, "No Code");
      else
	strcpy (codetype, "unknown code");
      break;
    case 16:
      strcpy (restype, "Information Reply");
      if (icmpcode == 0)
	strcpy (codetype, "No Code");
      else
	strcpy (codetype, "unknown code");
      break;
    case 17:
      strcpy (restype, "Address Mask Request");
      if (icmpcode == 0)
	strcpy (codetype, "No Code");
      else
	strcpy (codetype, "unknown code");
      break;
    case 18:
      strcpy (restype, "Address Mask Reply");
      if (icmpcode == 0)
	strcpy (codetype, "No Code");
      else
	strcpy (codetype, "unknown code");
      break;
    case 30:
      strcpy (restype, "Traceroute");
      if (icmpcode == 0)
	strcpy (codetype, "No Code");
      else
	strcpy (codetype, "unknown code");
      break;
    case 31:
      strcpy (restype, "Datagram Conversion Error");
      if (icmpcode == 0)
	strcpy (codetype, "No Code");
      else
	strcpy (codetype, "unknown code");
      break;
    case 32:
      strcpy (restype, "Mobile Host Redirect");
      if (icmpcode == 0)
	strcpy (codetype, "No Code");
      else
	strcpy (codetype, "unknown code");
      break;
    case 33:
      strcpy (restype, "IPv6 Where-Are-You");
      if (icmpcode == 0)
	strcpy (codetype, "No Code");
      else
	strcpy (codetype, "unknown code");
      break;
    case 34:
      strcpy (restype, "IPv6 I-Am-Here");
      if (icmpcode == 0)
	strcpy (codetype, "No Code");
      else
	strcpy (codetype, "unknown code");
      break;
    case 35:
      strcpy (restype, "Mobile Registration Request");
      if (icmpcode == 0)
	strcpy (codetype, "No Code");
      else
	strcpy (codetype, "unknown code");
      break;
    case 36:
      strcpy (restype, "Moblie Registration Reply");
      if (icmpcode == 0)
	strcpy (codetype, "No Code");
      else
	strcpy (codetype, "unknown code");
      break;
    case 40:
      strcpy (restype, "Photuris");
      switch (icmpcode)
	{
	case 0:
	  strcpy (codetype, "Reserved");
	  break;
	case 1:
	  strcpy (codetype, "unknown security parameters index");
	  break;
	case 2:
	  strcpy (codetype,
		  "valid security parameters, but authentication failed");
	  break;
	case 3:
	  strcpy (codetype,
		  "valid security parameters, but decryption failed");
	  break;
	default:
	  strcpy (codetype, "unknown code");
	}
      break;
    default:
      strcpy (restype, "Unknown ICMP Type???");
      strcpy (codetype, "unknown code");
    }
  strcpy (icmpst, restype);
  strcat (icmpst, "/");
  strcat (icmpst, codetype);
  return 0;
}
